// opencd.cpp : Defines the entry point for the application.
//

#include "stdafx.h"


#define VERSION			  MAKEWORD( 1, 0 );
#define MAX_BUFFER_LEN    0xff
#define REGKEY			  TEXT("SOFTWARE\\KenamicK Entertainment\\OpenCD\\")
#define KEYVAL			  TEXT("LastAction")
#define KEYVAL_VER		  TEXT("Version")
#define CD_OPENED		  0x00000001
#define CD_CLOSED		  0x00000000


int ExitError( DWORD dwError )
{
	TCHAR msg[MAX_BUFFER_LEN + 1];
	
	if ( dwError != 0 )
	{
		mciGetErrorString( dwError, msg, MAX_BUFFER_LEN );
		MessageBox( NULL, msg, "Whoops!", MB_OK | MB_ICONEXCLAMATION );

		return 1;
	}
	else
		return 0;
}


// read key from registry
int RegReadKey( HKEY hRoot, LPCTSTR lpKeyName, LPCTSTR lpSubKeyName, DWORD *dwValue )
{

	HKEY  hKeyResult;
	DWORD dwReturn = RegOpenKeyEx( hRoot, lpKeyName, 0, KEY_ALL_ACCESS, &hKeyResult );

	if ( dwReturn != ERROR_SUCCESS )
		return 0;

	// get value
	DWORD lpType;
	BYTE bytData[4];
	DWORD dwsize = 1024;
	dwReturn = RegQueryValueEx( hKeyResult, lpSubKeyName, NULL,
								&lpType, bytData, &dwsize );

	if ( dwReturn != ERROR_SUCCESS )
	{
		RegCloseKey( hKeyResult );
		return 0;
	}

	DWORD dwVal = MAKELONG( MAKEWORD( bytData[0], bytData[1] ), MAKEWORD( bytData[2], bytData[3] ) );
	
	*dwValue = dwVal;

	RegCloseKey( hKeyResult );

	return 1;
}



// read key from registry
int RegSetKey( HKEY hRoot, LPCTSTR lpKeyName, LPCTSTR lpSubKeyName, DWORD dwValue )
{

	HKEY  hKeyResult;
	DWORD dwDipos;
	//DWORD dwReturn = RegCreateKey( hRoot, lpKeyName, &hKeyResult );
	DWORD dwReturn = RegCreateKeyEx( hRoot, lpKeyName, 0, 
									 TEXT("REG_SZ"), 
									 REG_OPTION_VOLATILE,  /* very important ;-] */
									 KEY_ALL_ACCESS,
									 NULL,
									 &hKeyResult,
									 &dwDipos 
									 );


	if ( dwReturn != ERROR_SUCCESS )
		return 0;

	// set value
	dwReturn = RegSetValueEx( hKeyResult, lpSubKeyName, NULL,
								REG_DWORD, (const BYTE *) &dwValue, sizeof(DWORD) );

	if ( dwReturn != ERROR_SUCCESS )
	{
		RegCloseKey( hKeyResult );
		return 0;
	}

	RegCloseKey( hKeyResult );

	return 1;
}



BOOL isCanEject( DWORD dwDevID )
{

	// can eject ?
	MCI_GETDEVCAPS_PARMS mciCaps;

	mciCaps.dwCallback = 0;
	mciCaps.dwItem = MCI_GETDEVCAPS_CAN_EJECT ;
	mciCaps.dwReturn = 0;

	MCIERROR mcr = mciSendCommand( dwDevID, MCI_GETDEVCAPS, MCI_WAIT | MCI_GETDEVCAPS_ITEM,
					  			  (DWORD)(LPMCI_GETDEVCAPS_PARMS) &mciCaps );

	if ( ExitError( mcr ) )
		return FALSE;

	DWORD dwCanEject = mciCaps.dwReturn;
	if ( !dwCanEject )
	{
		MessageBox( NULL, TEXT("The media cannot be ejected !"), TEXT("Hmmm...!"), MB_OK | MB_ICONEXCLAMATION );
		return FALSE;
	}

	return TRUE;
	
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	

	DWORD Open_State = CD_CLOSED;
	
	
	// get current cd door state
	if ( RegReadKey( HKEY_CURRENT_USER, REGKEY, KEYVAL, &Open_State ) )
	{
		if ( Open_State > 0 )
			Open_State = CD_OPENED;
	}
	else
	{
		// write default
		RegSetKey( HKEY_CURRENT_USER, REGKEY, KEYVAL, Open_State );
		DWORD dwVer = VERSION;
		RegSetKey( HKEY_CURRENT_USER, REGKEY, KEYVAL_VER, dwVer );
	}



	MCIERROR mcr;
	DWORD dwDeviceID;
	DWORD dwReturn;

	// open cd-audio device
	MCI_OPEN_PARMS mciOpen;

	mciOpen.dwCallback       = 0 ;
    mciOpen.wDeviceID        = 0 ;
    mciOpen.lpstrDeviceType  = TEXT ("cdaudio") ;
    mciOpen.lpstrElementName = TEXT ("") ; 
    mciOpen.lpstrAlias       = NULL ;
	
	mcr = mciSendCommand( 0, MCI_OPEN, MCI_WAIT | MCI_OPEN_TYPE,
						  (DWORD)(LPMCI_OPEN_PARMS) &mciOpen );


	if ( ExitError( mcr ) )
		return -1;

	// get device ID
	dwDeviceID = mciOpen.wDeviceID;

	if ( ! isCanEject( dwDeviceID ) )
		goto the_end;


/*	// open or close
	MCI_SET_PARMS mciSet;

	mciSet.dwCallback = 0;
	mciSet.dwAudio	  = 0;
	mciSet.dwTimeFormat = 0;

	mcr = mciSendCommand( dwDeviceID, MCI_SET, MCI_WAIT | MCI_SET_DOOR_OPEN ,
						  (DWORD)(LPMCI_OPEN_PARMS) &mciSet );

	ExitError( mcr );
*/
	
	// get status
	MCI_STATUS_PARMS mciStatus;

	::ZeroMemory( &mciStatus, sizeof(mciStatus) );
	mciStatus.dwItem = MCI_STATUS_READY;

	mcr = mciSendCommand( dwDeviceID, 
						  MCI_STATUS,
						  MCI_WAIT | MCI_STATUS_ITEM ,
						  (DWORD)(LPMCI_STATUS_PARMS) &mciStatus );

	ExitError( mcr );

	// check if device should be opened or closed
	dwReturn = mciStatus.dwReturn;

	
	/*// check if there's media inside
	::ZeroMemory( &mciStatus, sizeof(mciStatus) );
	mciStatus.dwItem = MCI_STATUS_MEDIA_PRESENT ;

	mcr = mciSendCommand( dwDeviceID, 
						  MCI_STATUS,
						  MCI_WAIT | MCI_STATUS_ITEM ,
						  (DWORD)(LPMCI_STATUS_PARMS) &mciStatus );

	ExitError( mcr );
	
	if ( mciStatus.dwReturn & )
	{
		// a media is inside -> do nothing
	}
	else
	{
		dwReturn = FALSE;
	}*/


	if ( dwReturn )
	{
		// device is ready - there is a cd inside
		Open_State = CD_CLOSED;
	}
	else
	{
		// device is not ready - there is not a CD inside or device is opened
		// so use registry to find out last action
	}

	// perofrm action
	switch( Open_State )
	{
	case CD_OPENED:
		mcr = mciSendString( "SET CDAUDIO DOOR CLOSED WAIT", NULL, NULL, NULL );
		
		if ( ! ExitError( mcr ) )
			Open_State = CD_CLOSED;

		break;

	case CD_CLOSED:
		mcr = mciSendString( "SET CDAUDIO DOOR OPEN WAIT", NULL, NULL, NULL );

		if ( ! ExitError( mcr ) )
			Open_State = CD_OPENED;

		break;
	}


	// save last action to registry
	RegSetKey( HKEY_CURRENT_USER, REGKEY, KEYVAL, Open_State ); 


the_end:

	// close cd-audio device
	mcr = mciSendCommand( dwDeviceID, MCI_CLOSE, MCI_WAIT,
						  NULL );
	
	if ( ExitError( mcr ) )
		return -1;


	return 0;
}



